c----------------------------------------------------------------------
c --- FEPS2BAEM -- FEPS Output to BAEMARB.DAT Converter
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43           Level: 110225             MAIN
c
c     Copyright (c) 2011 by Exponent, Inc.
c
c --- Written by:  I. Lee   (FEPS2BAEM)
c
c --- PURPOSE:
c       FEPS2BAEM converts the output of the Forest Service's Fire  
c       Emission Production Simulator (FEPS) to a buoyant area source 
c       emissions file with arbitrarily varying emissions (BAEMARB.DAT) 
c       for input into CALPUFF.
c
c --- CONTROL FILE VERSION:
c       'FEPS2BAEM.INP   1.1'
c
c --- UPDATES:
c
c --- Version 1.42, Level 090511 to Version 1.43, Level 110225   (DGS)
c          Updated CALUTILS.FOR to version v2.58 (110225)
c          - Add control file variable type 5 (character array) and
c            retain commas in string returned so that array of
c            variable values can be parsed
c          Modified: READIN, ALTONU, SETVAR
c
c --- Version 1.41, Level 080407 to Version 1.42, Level 090511   (DGS)
c        - CALUTILS from v2.56 Level 080407 to v2.571 Level 090511
c          Increase control file line length to 200 characters
c          Activate CPU clock using F95 system routine
c          Add routine to reformat a date string
c          New     : FMT_DATE
c          Modified: PARAMS.CAL, READIN, DATETM
c        - Reformat date reported to list file
c          Modified: FIN
c
c     Version 1.4, Level 080211 to Version 1.41, Level 080407
c        - CALUTILS from v2.55 Level 070327 to v2.56 Level 080407
c          Control file entries in exponential notation were not correct
c          if decimal point was missing (2e-02 was read as 0.2e-02).
c          Modified: ALTONU
c
c     Version: 1.3_BETA Level: 070613 to Version 1.4, Level 080211 (IWL)
c       1. [COMP] Fix call to HFLAME to correctly use diameter and not
c          radius.
c       2. COORDLIB from v1.98 Level 060911 to v1.99 Level 070921
c          - Conversion of point in S. hemisphere to UTM-N returned coord.
c            as UTM-S instead, for conversions from all map projections
c            except lat/lon.
c          - Initialization of a few work arrays were missing.  These have
c            no effect on results.
c
c     Version: 1.2_BETA Level: 070608 to Version: 1.3_BETA Level: 070613 (IWL)
c       1. [COMP] Enable IFLAG option in control file.
c       2. [READCF/COMP] Enable 3 Comment Lines at top of control file,
c          now write them to top of BAEMARB.DAT file.
c       3. [PARAMS.F2B,QA.F2B,MAIN,SETUP] Add check of control file model name
c          and version number.
c       4. Add particle size distribution categories option for PM2.5 in
c          control file (variable = PMDIST)
c
c     Version: 1.1_BETA Level: 070530 to Version: 1.2_BETA Level: 070608 (IWL)
c       1. [COMP] Change hard-coded species name from "PM2P5" to "PM2.5".
c       2. [READCF] (Input Group 0b) Allow for spaces in the path/filenames 
c          of the FEPS Consumption/Emissions Report and Buoyancy Results
c          Report Files.
c       3. [READCF] (Input Group 0b) Clear out string variables between
c          reading in each set of fire variables.
c       4. [COMP] Fix GLOBE1 calls to fix input/output variable order,
c          was reversed.
c
c     Version: 1.0_BETA Level: 070529 to Version: 1.1_BETA Level: 070530 (IWL)
c       1. Allow for multiple fires with differing time spans.
c
c     Version: 1.0_BETA Level: 070522 to Version: 1.0_BETA Level: 070529 (IWL)
c       1. Update comment block at top of each subroutine to identify
c          the parameters and common block variables used as well as 
c          the calling and called subroutines.
c
c --- PARAMETERS USED:
c                FEPS2BAEM Parameters
c                iolst
c
c --- COMMON BLOCK VARIABLES USED:
c                /QA/ Common Block
c                ver, level, inpmod, inpver
c ---------------------------------------------------------------------
      Program FEPS2BAEM
c
c --- Include parameters
      include 'params.f2b'
c
c --- Include common blocks
      include 'qa.f2b'
c
c --- Set version and level number of program
      ver='1.43'
      level='110225'
c
c --- Set model and version number of control file
      inpmodel='FEPS2BAEM.INP'
      inpver='1.1'
c
c --- SETUP PHASE -- read control file information
      call SETUP
c
c --- COMPUTATIONAL PHASE -- process data files
      call COMP
c
c --- TERMINATION PHASE -- program termination functions
      call FIN
      close(iolst)
c
      stop
      end

c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43         Level: 070522         BLOCK DATA
c                I. Lee, TRC Env. Corp.
c
c --- Include parameter statements
      include 'params.f2b'

c --- Include common blocks
      include 'filnam.f2b'
      include 'control.f2b'
      include 'qa.f2b'

c --- FILNAM common block
      data runinp/'feps2baem.inp'/,f2blst/'feps2baem.lst'/,
     1     ardat/'baemarb.dat'/

c --- FILLOG common block
      data lcfiles/.true./

c --- CONTROL common block
c --- Primary variables
      data nfire/0/
      data pmap/'UTM     '/
c --- Derived variables
      data lutm/.false./, llcc/.false./, lps/.false./
      data lem/.false./, llaza/.false./, lttm/.false./
      data lacea/.false./

c --- QA common block
      data model/'FEPS2BAEM   '/

      end

c----------------------------------------------------------------------
c --- BRING IN CALPUFF SYSTEM UTILITY SUBROUTINES
      include 'calutils.for'
      include 'coordlib.for'
c----------------------------------------------------------------------

c----------------------------------------------------------------------
      subroutine SETUP
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43      Level: 070522                 SETUP
c                I. Lee, TRC Env. Corp.
c
c     PURPOSE:   SETUP reads and checks the control data provided, sets
c                logicals, echoes the control data, and opens the data
c                files if inputs are valid.
c
c --- UPDATES:
c
c --- PARAMETERS USED:
c                FEPS2BAEM Parameters
c                ioctl, iomsg, mver, mlevel
c
c --- COMMON BLOCK VARIABLES USED:
c                /QA/ Common Block
c                rcpu, rdate, rtime, ver, level
c
c                /FILNAM/ Common Block
c                runinp
c
c --- CALLED BY: MAIN
c
c --- CALLS:     DATETM, COMLINE, READCF
c                      
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.f2b'

c --- Include common blocks
      include 'filnam.f2b'
      include 'qa.f2b'

c --- Get date and time from system
      call DATETM(rdate,rtime,rcpu)

c --- Get the name of the control file from the command line
      call COMLINE(runinp)

c --- Open the control file
      open(ioctl,file=runinp,status='old')

c --- Get model name and version number of the control file
      read(ioctl,*)minpmodel,minpver

c --- Report progress
      write(iomsg,*)'SETUP PHASE'

c --- Check that the version and level number in the parameter
c --- file matches those in the code itself
      if(ver.ne.mver.or.level.ne.mlevel)then
         write(iomsg,10) ver,level,mver,mlevel
10       format(/1x,'ERROR in SUBR. SETUP -- The FEPS2BAEM version ',
     1   'and level numbers do not match those in the parameter file'/
     2   5x,'    Model Code - Version: ',a12,' Level: ',a12/
     3   5x,'Parameter File - Version: ',a12,' Level: ',a12)
         stop
      endif

c --- Check that the model name and version number of the control
c --- file matches those in the code itself
      if(inpmodel.ne.minpmodel.or.inpver.ne.minpver)then
         write(iomsg,20) inpmodel,inpver,minpmodel,minpver
20       format(/1x,'ERROR in SUBR. SETUP -- The FEPS2BAEM model ',
     1   'and version number do not match those in the control file'/
     2   5x,'  Model Code - ',a15,' Version: ',a12/
     3   5x,'Control File - ',a15,' Version: ',a12)
         stop
      endif

c --- Read control file (open other files)
      call READCF

      return
      end

c----------------------------------------------------------------------
      subroutine COMP
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43       Level: 080211                 COMP
c                I. Lee, TRC Env. Corp.
c
c --- PURPOSE:   Main computational routine
c
c --- UPDATES:
c
c     Version: 1.1_BETA Level: 070530 to Version: 1.2_BETA Level: 070608 (IWL)
c       1. [COMP] Change hard-coded species name from "PM2P5" to "PM2.5".
c       2. [COMP] Fix GLOBE1 calls to fix input/output variable order,
c          was reversed.
c
c --- PARAMETERS USED:
c                FEPS2BAEM Parameters
c                mxfire, mxspec, mxdur, iomsg, iolst, iobae
c
c --- COMMON BLOCK VARIABLES USED:
c                /FILNAM/ Common Block
c                ardat, fepsemis, fepsbuoy
c
c                /CONTROL/ Common Block
c                f2blbl, nfire, firename, firelat, firelon,
c                fireelev, firesigz, xmolwt, iflag, abtz,
c                lutm, llcc, lps, lem, llaza, lttm, pmap
c
c                /GRID/ Common Block
c                izone, reflat, reflon, xlat1, xlat2,
c                feast, fnorth, utmhem, datum, daten,
c                clat0, clon0, clat1, clat2
c
c                /QA/ Common Block
c                ver, level
c
c --- CALLED BY: MAIN
c
c --- CALLS:     GLOBE1, GLOBE, AREATOSQ, INDECR, GRDAY, INCR, DELTT,
c                PMDIST4
c
c----------------------------------------------------------------------
c
c --- include parameters
      include 'params.f2b'

c --- include common blocks
      include 'control.f2b'
      include 'filnam.f2b'
      include 'grid.f2b'
      include 'qa.f2b'

c --- Local variables
      character*16 dataset,dataver
      character*64 datamod
      character*80 comment1
      character*86 comment2
      character*4 c4dum
      character*132 qcevnt(mxfire)
      character*4 cbsec,cesec
      character*50 verdoc
      character*12 cspecs(mxfire)
      integer qibyr(0:mxdur,mxfire)
      integer qibjul(0:mxdur,mxfire)
      integer qibhr(0:mxdur,mxfire)
      integer qieyr(0:mxdur,mxfire)
      integer qiejul(0:mxdur,mxfire)
      integer qiehr(0:mxdur,mxfire)
      integer qidur(0:mxdur,mxfire)
      integer ndur
      integer idurtot(mxfire)
      integer ibyr,ibjul,ibhr
      integer ieyr,iejul,iehr
      integer ibmo,ibday,iemo,ieday
      integer imxbyr,imxbjul,imxbhr
      integer imxeyr,imxejul,imxehr
      integer index(mxspec)
      integer idelthr
      integer ibegin(mxfire),iend(mxfire)
      integer irec
      real zerotmp
      real qxctr(mxfire),qyctr(mxfire)
      real qxvert(0:mxdur,4,mxfire),qyvert(0:mxdur,4,mxfire)
      real qradi(0:mxdur,mxfire)
      real qxheat(0:mxdur,mxfire)
      real qxflux(0:mxdur,mxfire)
      real qxtambk(0:mxdur,mxfire)
      real qxarea(0:mxdur,mxfire)
      real qemit(0:mxdur,mxspec,mxfire)
      real qxeffht(0:mxdur,mxfire)
      real qxeffw(0:mxdur,mxfire)
      real tfirek,ramb,cplume,gravi
      real subcats4(4)
      real diam
c --- For coordinate transformations
      real*8 vecti(9),vecto(9)
      character*12 caction
      logical ltrans
      character*8 datumll

c --- For QA output
      logical lqa

c --- Initialize values
      qcevnt=' '
      qibyr=0
      qibjul=0
      qibhr=0
      qieyr=0
      qiejul=0
      qiehr=0
      qidur=0
      ndur=0
      idurtot=0
      qxvert=0.
      qyvert=0.
      qradi=0.
      qemit=0.
      qxeffht=0.
      qxeffw=0.
      idelthr=0
      zerotmp=0.

c --- Set values
c --- Assume begin seconds = 0 and end seconds = 3600
      cbsec='0000'
      cesec='3600'
c --- Assumed flame temperature (K)
      data tfirek/900./
c --- Assumed ambient density (kg/m**3, @ 285 K and 1 atm)
      data ramb/1.24/
c --- Plume specific heat (kJ/kg*K)
      data cplume/1.004/
c --- Gravitational acceleration (m/s**2)
      data gravi/9.81/
c --- Datum of input LL coords.
      data datumll/'WGS-84  '/

c --- Set species names
      cspecs(1)='CO'
      cspecs(2)='CH4'
      cspecs(3)='PM2.5'
      cspecs(4)='PM1.87'
      cspecs(5)='PM1.12'
      cspecs(6)='PM0.81'
      cspecs(7)='PM0.56'

c --- Configure output variables
      data dataset/'BAEMARB.DAT'/, dataver/'2.1'/
      data datamod
     1 /'Comments, times with seconds, time zone, coord info'/
      data ncomment/6/
      data comment2/'Produced by FEPS2BAEM Version: '/

c --- Construct the version-level comment string
      j=32
      do i=1,12
         if(ver(i:i).NE.' ') then
            comment2(j:j)=ver(i:i)
            j=j+1
         endif
      enddo
      j=j+1
      comment2(j:j+7)=' Level: '
      j=j+8
      do i=1,12
         if(level(i:i).NE.' ') then
            comment2(j:j)=level(i:i)
            j=j+1
         endif
      enddo

c --- Set dummy values
      data xdum/0.0/,rdum/0.0/
      data idum/0/

c --- Report progress
      write(iomsg,*) 'COMPUTATIONAL PHASE'
      write(iolst,*)
      write(iolst,*) '--------------------------'
      write(iolst,*) '    PROCESS Information'
      write(iolst,*) '--------------------------'
      write(iolst,*)

c --- Report grid projection information
      write(iolst,*) 'Output Map Projection'
      write(iolst,*) ' Datum: ' ,datum
      write(iolst,*) ' Projection: ' ,pmap
      if(LUTM) then
         write(iolst,*) ' Hemisphere: ',utmhem
         write(iolst,*) ' UTM Zone: ' ,izone
      endif
      if(LLCC.or.LPS.or.LEM.or.LLAZA.or.LTTM) then
         write(iolst,*) ' Origin Latitude (N): ' ,reflat
         write(iolst,*) ' Origin Longitude (E): ' ,reflon
         if(LLCC.or.LPS)write(iolst,*)
     &     ' Matching Parallel 1: ' ,xlat1
         if(LLCC)write(iolst,*)
     &     ' Matching Parallel 2: ' ,xlat2
      endif
      if(LLCC.or.LLAZA.or.LTTM) then
         write(iolst,*) ' False Easting (km): ' ,feast
         write(iolst,*) ' False Northing (km): ' ,fnorth
      endif
      write(iolst,*)
      write(iolst,*)

c --- Report to list file
      write(iolst,*)'Number of fires processed: ',nfire
      write(iolst,*)
      write(iolst,*)

c --- Set translation vectors from LL to the output grid
      if(lutm) then
c ---   Using UTM grid
        call GLOBE1('LL      ',idum,xdum,xdum,xdum,rdum,rdum,
     &             feasti,fnorti,
     &             'UTM     ',iutm,xdum,xdum,xdum,rdum,rdum,
     &             feast,fnorth,
     &             caction,vecti,vecto)
      elseif(llcc) then
c ---  Using Lambert Conformal grid
       call GLOBE1('LL      ',idum,xdum,xdum,xdum,rdum,rdum,
     &             feasti,fnorti,
     &             'LCC      ',idum,xdum,xlat1,xlat2,reflat,reflon,
     &             feast,fnorth,
     &             caction,vecti,vecto)
      elseif(lps) then
c ---  Using Polar Stereographic grid
       call GLOBE1('LL      ',idum,xdum,xdum,xdum,rdum,rdum,
     &             feasti,fnorti,
     &             'PS      ',idum,xdum,xlat1,xdum,reflat,reflon,
     &             feast,fnorth,
     &             caction,vecti,vecto)
      elseif(lem) then
c ---  Using Equatorial Mercator grid
       call GLOBE1('LL      ',idum,xdum,xdum,xdum,rdum,rdum,
     &             feasti,fnorti,
     &             'EM      ',idum,xdum,xlat1,-xlat1,reflat,reflon,
     &             feast,fnorth,
     &             caction,vecti,vecto)
      elseif(llaza) then
c ---  Using Lambert Azimuthal Equal Area grid
       call GLOBE1('LL      ',idum,xdum,xdum,xdum,rdum,rdum,
     &             feasti,fnorti,
     &             'LAZA    ',idum,xdum,xdum,xdum,reflat,reflon,
     &             feast,fnorth,
     &             caction,vecti,vecto)
      elseif(lttm) then
c ---  Using Tangential TM grid
       call GLOBE1('LL      ',idum,xdum,xdum,xdum,rdum,rdum,
     &             feasti,fnorti,
     &             'TM      ',idum,tmsone,xdum,xdum,reflat,reflon,
     &             feast,fnorth,
     &             caction,vecti,vecto)
      endif

c --- Modify IFLAG and set XMOLWT based on IPMDIST value
c --- Four size categories
      if(ipmdist.eq.1) then
        if(iflag(3).eq.1) then
c ---     Set PM2.5 flag to 0 and four sub-categories to 1
c ---     Set XMOLWT for the four sub-categories to XMOLWT of PM2.5
          iflag(3)=0
          do i=4,7
            iflag(i)=1
            xmolwt(i)=xmolwt(3)
          enddo
        endif
      endif

c --- Construct index array for the species to be considered.
c --- For example, if species 1, 2, and 4 are to be considered,
c --- then index(1)=1, index(2)=2, and index(3)=4.
      nspec=0
      do i=1,mxspec
        if (iflag(i).eq.1) then
          nspec=nspec+1
          index(nspec)=i
        end if
      end do

c ---------------------------------------
c --- Read in data from FEPS Report files
c --- Loop over number of fires
      do ifire=1,nfire
        call RDFEPS(fepsemis(ifire),fepsbuoy(ifire),ifire,nspec,
     &       qieyr,qiejul,qiehr,qidur,idurtot(ifire),qcevnt(ifire),
     &       qxheat,qxflux,qxarea,qxtambk,qemit)
      enddo

c --- Initialize max begin time with begin time of first fire
      imxbyr=qieyr(1,1)
      imxbjul=qiejul(1,1)
      imxbhr=qiehr(1,1)
c --- HARDCODED TO ONE-HOUR TIMESTEPS
      call INDECR(iolst,imxbyr,imxbjul,imxbhr,-1,0,23)

c --- Initialize max end time with end time of first fire
      imxeyr=qieyr(idurtot(1),1)
      imxejul=qiejul(idurtot(1),1)
      imxehr=qiehr(idurtot(1),1)

c ------------------------
c --- Perform computations
c --- Loop over number of fires
      do ifire=1,nfire

c ---   Convert coords. of center of fire from lat/lon to grid X/Y
	call GLOBE(iolst,caction,datumll,vecti,datum,vecto,
     &           firelon(ifire),firelat(ifire),
     &           qxctr(ifire),qyctr(ifire),idum,c4dum)

c ---   Loop over each hour of each fire
        do i=1,idurtot(ifire)

c ---     Get begin time of each record by subtracting one hour from record end time
          qibyr(i,ifire)=qieyr(i,ifire)
          qibjul(i,ifire)=qiejul(i,ifire)
          qibhr(i,ifire)=qiehr(i,ifire)

          call INDECR(iolst,qibyr(i,ifire),qibjul(i,ifire),
     &                qibhr(i,ifire),-1,0,23)

c ---     Convert ambient air temperature from degC to K
          qxtambk(i,ifire)=qxtambk(i,ifire)+273.15

c ---     Calculate vertices of area as a square (m**2) and source radius
          call AREATOSQ(qxarea(i,ifire),qxctr(ifire),qyctr(ifire),
     &             qxvert(i,1:4,ifire),qyvert(i,1:4,ifire),
     &             qradi(i,ifire))

c ---     Convert plume heat release from kJ/hr to kj/s (i.e. kW)
          qxheat(i,ifire)=qxheat(i,ifire)/3600.

c ---     Calculate effective height of emissions (i.e. flame height)
c ---     If source diameter = 0, then set eff. height = 0
          if(qradi(i,ifire).NE.0.) then
c ---       Compute diameter from radius
            diam=2*qradi(i,ifire)
            call HFLAME(qxeffht(i,ifire),qxheat(i,ifire),
     &             diam,ramb,qxtambk(i,ifire),cplume,gravi)
          else
            qxeffht(i,ifire)=0.
          endif

c ---     Calculate plume vertical speed
c ---     If source diameter = 0, then set eff. height = 0
          if(qradi(i,ifire).NE.0.) then
            call EFFWP(qxflux(i,ifire),qxtambk(i,ifire),
     &             qradi(i,ifire),tfirek,gravi,qxeffw(i,ifire))
          else
            qxeffw(i,ifire)=0.
          endif

c ---     Calculate emission rates of PM2.5 sub-size categories if applicable
          if(ipmdist.eq.1) then
            call PMDIST4(qemit(i,3,ifire),4,subcats4)
            do j=4,7
              qemit(i,j,ifire)=subcats4(j-3)
            enddo
          endif

        enddo

c ---   Get maximum start time across all fires
c ---   Get difference between current max begin time and begin time of current fire
        idelthr=0
        call DELTT(qibyr(1,ifire),qibjul(1,ifire),qibhr(1,ifire),
     &             imxbyr,imxbjul,imxbhr,idelthr)
c ---   If the difference is positive, then replace the current max begin time with
c ---   the begin time of the current fire
        if(idelthr.GT.0) then
          imxbyr=qibyr(1,ifire)
          imxbjul=qibjul(1,ifire)
          imxbhr=qibhr(1,ifire)
        endif

c ---   Get maximum end time across all fires
c ---   Get difference between current max begin time and begin time of current fire
        idelthr=0
        call DELTT(qieyr(1,ifire),qiejul(1,ifire),qiehr(1,ifire),
     &             imxeyr,imxejul,imxehr,idelthr)
c ---   If the difference is positive, then replace the current max end time with
c ---   the end time of the current fire
        if(idelthr.GT.0) then
          imxeyr=qieyr(idurtot(ifire),ifire)
          imxejul=qiejul(idurtot(ifire),ifire)
          imxehr=qiehr(idurtot(ifire),ifire)
        endif

c ---   Get total duration across all fires
        call DELTT(imxbyr,imxbjul,imxbhr,imxeyr,imxejul,imxehr,ndur)

c ------------------------------
c ---   Report back to LIST file
c ---   Fire number
        write(iolst,*) 'FIRE NUMBER: ',ifire

c ---   Fire name
        write(iolst,*) '  Name: ',firename(ifire)

c ---   X/Y center of each fire
        write(iolst,'(a18,f12.4)') 'Center X (km): ',qxctr(ifire)
        write(iolst,'(a18,f12.4)') 'Center Y (km): ',qyctr(ifire)

c ---   Fire Elevation
        write(iolst,'(a18,f12.4)')'Elevation (m): ',
     &                 fireelev(ifire)

c ---   Fire initial vertical spread
        write(iolst,'(a32,f12.4)') 'Initial Vertical Spread (m): ',
     &                 firesigz(ifire)

c ---   FEPS Event name
        write(iolst,*) '  FEPS Event Name: ',qcevnt(ifire)

c ---   FEPS Files used
        write(iolst,*) '  FEPS Consumption/Emissions Report: ',
     &                 fepsemis(ifire)
        write(iolst,*) '  FEPS Buoyancy Results Report: ',
     &                 fepsbuoy(ifire)

c ---   Fire time span
c ---   Convert begin time to Gregorian Date
        call GRDAY(iolst,qibyr(1,ifire),qibjul(1,ifire),ibmo,ibday)
        write(iolst,*) '  Start Time: ',qibyr(1,ifire),' ',
     &    ibmo,' ',ibday,' ',qibhr(1,ifire),' ',cbsec

c ---   Convert end time to Gregorian Date
        call GRDAY(iolst,qieyr(idurtot(ifire),ifire),
     &             qiejul(idurtot(ifire),ifire),iemo,ieday)
        write(iolst,*) '  End Time: ',qieyr(idurtot(ifire),ifire),
     &    ' ',iemo,' ',ieday,' ',qiehr(idurtot(ifire),ifire),' ',cesec

c ---   Duration of fire
        write(iolst,*) '  Duration (hrs): ',idurtot(ifire)

        write(iolst,*)
      enddo

c --- Get begin and end time index for each fire
c --- Loop over fires
      do ifire=1,nfire
        call DELTT(imxbyr,imxbjul,imxbhr,qibyr(1,ifire),
     &    qibjul(1,ifire),qibhr(1,ifire),ibegin(ifire))
        call DELTT(qieyr(idurtot(ifire),ifire),
     &    qiejul(idurtot(ifire),ifire),qiehr(idurtot(ifire),ifire),
     &    imxeyr,imxejul,imxehr,iend(ifire))
        ibegin(ifire)=ibegin(ifire)+1
        iend(ifire)=ndur-iend(ifire)
      enddo

c --------------------------
c --- Write BAEMARB.DAT file

c --- Open BAEMARB.DAT file
      open (unit=iobae,file=ardat,status='unknown')

c --- Obtain COORDS version information
      call COORDSVER(iolst,verdoc)

c --- Write header records
c --- Record 1:  Dataset, Version, Modifier
      write(iobae,'(2a16,a64)') dataset,dataver,datamod
c --- Record 2:  Number of comment records
      write(iobae,'(i4)') ncomment
c --- Record 3:  Comment (optional/repeatable)
      write(iobae,'(a86)') comment2
c --- Record 4: Report COORDS version
      comment2(1:36)='Internal Coordinate Transformations '
      comment2(37:86)=verdoc
      write(iobae,'(a86)') comment2
c --- Record 5-7: 3 Comment Lines from control file
      do i=1,3
        write(iobae,'(a80)') comm(i)
      enddo
c --- Record 8:  Comment (data file label)
      write(iobae,'(a80)') f2blbl
c --- Record 9:  Map projection
      write(iobae,'(a8)') pmap
c --- Record 10:  Map projection parameters
      if(LUTM) then
         write(iobae,'(i4,a4)') izone,utmhem
      elseif(LLCC) then
         write(iobae,'(4a16)') clat0,clon0,clat1,clat2
      elseif(LPS) then
         write(iobae,'(3a16)') clat0,clon0,clat1
      elseif(LEM.or.LLAZA.or.LTTM) then
         write(iobae,'(2a16)') clat0,clon0
      endif
c --- Record 11:  False Easting, Northing
      if(LLCC.or.LLAZA.or.LTTM) then
         write(iobae,*) feast,fnorth
      endif
c --- Record 12:  Map DATUM
      call NIMADATE(daten)
      write(iobae,'(a8,a12)') datum,daten
c --- Record 13:  XYUNIT
      write(iobae,'(a4)') 'KM  '
c --- Record 14:  UTC Base Time Zone
      write(iobae,'(a8)') abtz

c --- Write time-invariant records

c --- Begin/End time
c --- Write record
      write(iobae,10)imxbyr,imxbjul,imxbhr,cbsec,
     &               imxeyr,imxejul,imxehr,cesec
10    format(i4,1x,i3,1x,i2,1x,a4,1x,i4,1x,i3,1x,i2,1x,a4)

c --- Number of fires and number of species
      write(iobae,'(2i4)')nfire,nspec

c --- Species names - HARDCODED TO CURRENT FEPS OUTPUT
      write (iobae,*)('''',TRIM(ADJUSTL(cspecs(index(i)))),
     &                ''' ',i=1,nspec)

c --- Species molecular weights
      write(iobae,*)(xmolwt(index(i)),i=1,nspec)

c --- Source information - HARDCODED TO g/s
c --- Loop over fires
      do ifire=1,nfire
        write(iobae,30)'''',firename(ifire),'''','''','g/s','''',
     &             '0.0','0.0'
30      format(a1,a16,a1,3x,a1,a3,a1,2(5x,a3))
      enddo

c --- Initialize time to overall begin time
      ibyr=imxbyr
      ibjul=imxbjul
      ibhr=imxbhr

c --- Loop over total number of hours
      do i=1,ndur

c ---   Write begin/end time of record - HARDCODED TO ONE-HOUR TIMESTEPS
        write(iobae,40)ibyr,ibjul,ibhr,cbsec,ibyr,ibjul,ibhr,cesec
40      format(2(i11,i5,i3,2x,a4))

c ---   Loop over fires
        do ifire=1,nfire
c ---     If fire exists for the current timestep, then write record
          if(ibegin(ifire).LE.i .AND. iend(ifire).GE.i) then
c ---       Set index to loop hour plus begin index of fire
            irec=i-ibegin(ifire)+1
            write(iobae,50)'''',firename(ifire),'''',
     &        qxvert(irec,1,ifire),qxvert(irec,2,ifire),
     &        qxvert(irec,3,ifire),qxvert(irec,4,ifire)
            write(iobae,60)qyvert(irec,1,ifire),qyvert(irec,2,ifire),
     &        qyvert(irec,3,ifire),qyvert(irec,4,ifire)
            write(iobae,70)qxeffht(irec,ifire),fireelev(ifire),tfirek,
     &        qxeffw(irec,ifire),qradi(irec,ifire),firesigz(ifire)
            write(iobae,80)(qemit(irec,index(j),ifire),j=1,nspec)
          else
c ---     Write a zero record for fire
            write(iobae,50)'''',firename(ifire),'''',zerotmp,
     &        zerotmp,zerotmp,zerotmp
            write(iobae,60)zerotmp,zerotmp,
     &        zerotmp,zerotmp,zerotmp
            write(iobae,70)zerotmp,zerotmp,zerotmp,
     &        zerotmp,zerotmp
            write(iobae,80)(zerotmp,j=1,nspec)
          endif

50        format(a1,a16,a1,4(1x,f12.4))
60        format(18x,4(1x,f12.4))
70        format(18x,6(1x,f12.4))
80        format(18x,6(1x,f12.4))

        enddo

c --- Increment time by one hour
      call INDECR(iolst,ibyr,ibjul,ibhr,1,0,23)

      enddo

      close(iobae)

      return
      end

c----------------------------------------------------------------------
      subroutine READCF
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43        Level: 070608              READCF
c                I. Lee, TRC Env. Corp.
c
c --- PURPOSE:   Read the file containing the file names of the
c                input and output files of the run, and associated
c                control variables, place into program variables,
c                and QA information.
c
c --- UPDATES:
c
c     Version: 1.1_BETA Level: 070530 to Version: 1.2_BETA Level: 070608 (IWL)
c       1. [READCF] (Input Group 0b) Allow for spaces in the path/filenames 
c          of the FEPS Consumption/Emissions Report and Buoyancy Results
c          Report Files.
c       2. [READCF] (Input Group 0b) Clear out string variables between
c          reading in each set of fire variables.
c
c --- PARAMETERS USED:
c                FEPS2BAEM Parameters
c                mxfire, mxspec, ioctl, iolst, iomsg
c
c                CALUTILS Parameters
c                mxvar
c
c --- COMMON BLOCK VARIABLES USED:
c                /FILNAM/ Common Block
c                f2blst, ardat, fepsemis, fepsbuoy, lcfiles
c
c                /CONTROL/ Common Block
c                comm, f2blbl, nfire, firename, firelat, firelon,
c                fireelev, firesigz, xmolwt, iflag, abtz,
c                lutm, llcc, lps, lem, llaza, lttm, pmap
c
c                /GRID/ Common Block
c                izone, reflat, reflon, xlat1, xlat2,
c                feast, fnorth, utmhem, datum,
c                clat0, clon0, clat1, clat2
c
c                /QA/ Common Block
c                ver, level
c
c --- CALLED BY: SETUP
c
c --- CALLS:     READIN, FILCASE, COORDSVER, XTRACTLL
c                      
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.f2b'
      include 'params.cal'
c
c --- Include common blocks
      include 'control.f2b'
      include 'filnam.f2b'
      include 'grid.f2b'
      include 'qa.f2b'

c --- Local variables
      character*4 ctemp132(132,2),ctemp80(80),ctemp16(16,3),ctemp8(8)
      character*4 clatlon(16,4)
      character*4 cpmap(8),cdatum(8)
      character*12 cvdic(mxvar,4)
      character*50 verdoc
      character*132 dbfil1,dbfil2,cblnk132
      character*16 ctmp1,ctmp2,ctmp3,cblnk16
      real xtmp1,xtmp2,xtmp3,xtmp4
      integer ivleng(mxvar,4),ivtype(mxvar,4)
      logical lecho
      logical lerrcf

c --- Set Dictionary
      data lecho/.false./
      data names/2/

c --- Read 3 comment lines
      read(ioctl,*)
      do i=1,3
        read(ioctl,'(a80)')comm(i)
      enddo

c --- Initialize
      do j=1,132
         cblnk132(j:j)=' '
      enddo
      cblnk16='                '

c --- Define data group dictionary
       data cvdic/
     a  'F2BLST','ARDAT','F2BLBL','ABTZ','NFIRE','LCFILES',54*' ',
     2  'FIRENAME','FEPSEMIS','FEPSBUOY','FIRELAT','FIRELON',
     2  'FIREELEV','FIRESIGZ',53*' ',
     b  'PMAP','IUTMZN','UTMHEM','RLAT0','RLON0','RLAT1','RLAT2',
     b  'DATUM','FEAST','FNORTH',50*' ',
     c  'XMOLWT','IFLAG','IPMDIST',57*' '/

      data ivleng/
     a  2*132, 80, 8, 2*1, 54*0,
     2  16,2*132, 2*16, 2*1, 53*0,
     b  8, 2*1, 4*16, 8, 2*1, 50*0,
     c  2*mxspec, 1, 57*0/

c --- Variable types (ivtype) are coded as:
c          0 = null
c          1 = real
c          2 = integer
c          3 = logical
c          4 = character
      data ivtype/
     a  4*4, 2, 3, 54*0,
     2  5*4, 2*1, 53*0,
     b  4, 2, 6*4, 2*1, 50*0,
     c  1, 2*2, 57*0/

c ------------------
c --- Input Group 0
c ------------------

c --- Initialize the temporary arrays
      do i=1,names
         do j=1,132
            ctemp132(j,i)(1:1)=' '
         enddo
      enddo
      do i=1,80
         ctemp80(i)(1:1)=' '
      enddo
      do i=1,8
         ctemp8(i)(1:1)=' '
      enddo

c --- Read the group data
       call READIN(cvdic(1,1),ivleng(1,1),ivtype(1,1),ioctl,iomsg,
     & lecho,
     1 ctemp132(1,1),ctemp132(1,2),ctemp80,ctemp8,NFIRE,LCFILES,
     2 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     3 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6 idum,idum,idum,idum,idum,idum)

c --- Prepare any filenames included in the I/O file by erasing
c --- the default filename set above
      if(ctemp132(1,1)(1:1).ne.' ')f2blst=' '
      if(ctemp132(1,2)(1:1).ne.' ')ardat=' '

c --- Transfer the char*4 data into the char*132 variables
      do j=1,132
         if(ctemp132(j,1)(1:1).ne.' ')f2blst(j:j)=ctemp132(j,1)(1:1)
         if(ctemp132(j,2)(1:1).ne.' ')ardat(j:j)=ctemp132(j,2)(1:1)
      enddo

c --- Transfer the char*4 data into the char*80 variable
      do j=1,80
         if(ctemp80(j)(1:1).ne.' ')f2blbl(j:j)=ctemp80(j)(1:1)
      enddo

c --- Transfer the char*4 data into the char*8 variable
      do j=1,8
         if(ctemp8(j)(1:1).ne.' ')abtz(j:j)=ctemp8(j)(1:1)
      enddo

c --- Convert the file names to the proper case
      call FILCASE(lcfiles,f2blst)
      call FILCASE(lcfiles,ardat)

c --- Open listfile
      open(iolst,file=f2blst,status='unknown')

c --- Write banner to list file
      write(iolst,5) ver,level
5     format(///,26x,'FEPS2BAEM OUTPUT SUMMARY',/,19x,'VERSION:  ',A12,
     1       ' LEVEL:  ',A12///)

c --- Obtain COORDS version information
      call COORDSVER(iolst,verdoc)
      write(iolst,*)'Internal Coordinate Transformations by ',verdoc
      write(iolst,*)
      write(iolst,*)
      write(iolst,*)

c --------------------
c --- Input Group 0(b)
c --------------------

c --- Loop over number of fires 
      do k=1,nfire
        do i=1,2
           do j=1,132
              ctemp132(j,i)(1:1)=' '
           enddo
        enddo
        do i=1,3
           do j=1,16
              ctemp16(j,i)(1:1)=' '
           enddo
        enddo

c ---   Read the group data
        call READIN(cvdic(1,2),ivleng(1,2),ivtype(1,2),ioctl,iomsg,
     &  lecho,
     1  ctemp16(1,1),ctemp132(1,1),ctemp132(1,2),ctemp16(1,2),
     2  ctemp16(1,3),xtmp3,xtmp4,
     2  idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     3  idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4  idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5  idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     7  idum,idum,idum,idum,idum)

c ---   Process if arrays have room
        if(k.LE.mxfire) then

c ---      Clear out variables and fill with blanks
           dbfil1=cblnk132
           dbfil2=cblnk132
           ctmp1=cblnk16
           ctmp2=cblnk16
           ctmp3=cblnk16

c ---      Transfer the char*4 data into the char*132 variables
           do j=1,132
              dbfil1(j:j)=ctemp132(j,1)(1:1)
              dbfil2(j:j)=ctemp132(j,2)(1:1)
           enddo

c ---      Transfer the char*4 data into the char*16 variables
           do j=1,16
              if(ctemp16(j,1)(1:1).ne.' ')ctmp1(j:j)=ctemp16(j,1)(1:1)
              if(ctemp16(j,2)(1:1).ne.' ')ctmp2(j:j)=ctemp16(j,2)(1:1)
              if(ctemp16(j,3)(1:1).ne.' ')ctmp3(j:j)=ctemp16(j,3)(1:1)
           enddo

c ---      Convert the filenames to the proper case
           call FILCASE(lcfiles,dbfil1)         
           call FILCASE(lcfiles,dbfil2)         

c ---      Translate character lat/lon to real NLat/ELon
           if(ctmp2(1:1).NE.' ')call XTRACTLL(iolst,'LAT ',ctmp2,xtmp1)
           if(ctmp3(1:1).NE.' ')call XTRACTLL(iolst,'LON ',ctmp3,xtmp2)

c ---      Place information in DB array, up to MXFIRE
c ---      Remove leading and trailing spaces in dbfil1 and dbfil2
           firename(k)=ctmp1
           fepsemis(k)=TRIM(ADJUSTL(dbfil1))
           fepsbuoy(k)=TRIM(ADJUSTL(dbfil2))
           firelat(k)=xtmp1
           firelon(k)=xtmp2
           fireelev(k)=xtmp3
           firesigz(k)=xtmp4

        endif

      enddo

c ------------------
c --- Input Group 1
c ------------------

c --- Initialize the temporary arrays for the character lat/lon fields
      do i=1,4
         do j=1,16
            clatlon(j,i)(1:1)=' '
         enddo
      enddo
      do j=1,16
         clat0(j:j)=' '
         clon0(j:j)=' '
         clat1(j:j)=' '
         clat2(j:j)=' '
      enddo

c --- Initialize the temporary array for the Datum-Region name and 
c --- map projection
      do j=1,8
         cpmap(j)(1:1)=' '
         cdatum(j)(1:1)=' '
      enddo

c --- Initialize input false easting and northing with defaults
      feastin=feast
      fnorthin=fnorth

      call READIN(cvdic(1,3),ivleng(1,3),ivtype(1,3),ioctl,iolst,lecho,
     1 CPMAP,IUTMZN,UTMHEM,
     2 CLATLON(1,1),CLATLON(1,2),CLATLON(1,3),CLATLON(1,4),
     3 CDATUM,FEASTIN,FNORTHIN,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     7 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     8 idum,idum)

c --- Transfer the char*4 data into the char*16 variables
      do j=1,16
         if(clatlon(j,1)(1:1).ne.' ')clat0(j:j)=clatlon(j,1)(1:1)
         if(clatlon(j,2)(1:1).ne.' ')clon0(j:j)=clatlon(j,2)(1:1)
         if(clatlon(j,3)(1:1).ne.' ')clat1(j:j)=clatlon(j,3)(1:1)
         if(clatlon(j,4)(1:1).ne.' ')clat2(j:j)=clatlon(j,4)(1:1)
      enddo

c --- Transfer the char*4 data into the char*8 variables
      if(cpmap(1)(1:1).ne.' ') then
         do j=1,8
            pmap(j:j)=cpmap(j)(1:1)
         enddo
      endif
      if(cdatum(1)(1:1).ne.' ') then
         do j=1,8
            datum(j:j)=cdatum(j)(1:1)
         enddo
      endif

c --- Pad the char*4 UTM Hemisphere
      utmhem(2:4)='   '

c ----------------------------------------------------
c --- Translate selected inputs to FEPS2BAEM variables
c ----------------------------------------------------

c --- Grid variables
      izone=iutmzn

c --- Translate character lat/lon to real NLat/ELon
      if(clat0(1:1).NE.' ') call XTRACTLL(iolst,'LAT ',clat0,reflat)
      if(clon0(1:1).NE.' ') call XTRACTLL(iolst,'LON ',clon0,reflon)
      if(clat1(1:1).NE.' ') call XTRACTLL(iolst,'LAT ',clat1,xlat1)
      if(clat2(1:1).NE.' ') call XTRACTLL(iolst,'LAT ',clat2,xlat2)

c --- Set logicals for map projection PMAP
      if(pmap.EQ.'UTM     ')  lutm =.TRUE.
      if(pmap.EQ.'LCC     ')  llcc =.TRUE.
      if(pmap.EQ.'PS      ')   lps =.TRUE.
      if(pmap.EQ.'EM      ')   lem =.TRUE.
      if(pmap.EQ.'LAZA    ') llaza =.TRUE.
      if(pmap.EQ.'TTM     ')  lttm =.TRUE.

c --- Adjust projection information if needed
      if(LEM) then
c ---    Equatorial Mercator projection matches at 0.0N, 
c ---    and places the northing origin at 0.0N
         reflat=0.0
         xlat1=0.0
         xlat2=0.0
      endif

c --- Transfer input false easting and northing if the projection
c --- can use it
      if(LLCC.or.LTTM.or.LLAZA) then
         feast=feastin
         fnorth=fnorthin
      endif

c ------------------
c --- Input Group 2
c ------------------

c     Read molecular weight and processing flag for each species
       call READIN(cvdic(1,4),ivleng(1,4),ivtype(1,4),ioctl,iomsg,
     & lecho,
     1 XMOLWT,IFLAG,IPMDIST,
     2 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     3 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     7 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum)

      return
      end

c----------------------------------------------------------------------
      subroutine RDFEPS(fepsemis1,fepsbuoy1,ifire,nspec,
     &                 qiyr,qijul,qihr,qidur,idurtot,cevnt,
     &                 qxheat,qxflux,qxarea,qxtambk,qemit)
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43         Level: 070522             RDFEPS
c                I. Lee, TRC Envir. Corp.
c
c --- PURPOSE:   Reads data from FEPS Consumption/Emissions Report and
c                FEPS Buoyancy Report
c
c --- INPUTS:
c      FEPSEMIS1 - character  - FEPS Consumption/Emissions Results Report filename
c      FEPSBUOY1 - character  - FEPS Buoyancy Report Results filename
c          IFIRE - integer    - fire no. to be processed
c          NSPEC - integer    - no. of species considered
c
c --- OUTPUTS:
c           QIYR - integer    - year (4-digit) of fire at each hour
c          QIJUL - integer    - Julian day of fire at each hour
c           QIHR - integer    - hour of fire  at each hour (LST, 0-23)
c          QIDUR - integer    - running fire duration (hours) at each hour
c        IDURTOT - integer    - total duration (hours) of fire
c          CEVNT - char*132   - event name
c         QXHEAT - real       - hourly plume heat release (kJ/hr)
c         QXFLUX - real       - hourly plume buoyancy flux (m**4/s**3)
c         QXAREA - real       - hourly fire size (hectares)
c        QXTAMBK - real       - hourly ambient temperature (K)
c          QEMIT - real       - hourly emission rates for species (g/s)
c                               (already packed according to the INDEX array)
c
c --- PARAMETERS USED:
c                FEPS2BAEM Parameters
c                mxfire, mxspec, mxdur, iofep1, iofep2, iolst, iomsg
c
c --- COMMON BLOCK VARIABLES USED:
c                None
c
c --- CALLED BY: COMP
c
c --- CALLS:     PARSE, JULDAY, RGET132
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.f2b'

c --- Local variables
      character*132 ctmp,blnk
      character*132 qctmp20(20),qctmp3(3)
      character*132 cevnt,cevnt1,cevnt2
      character*132 fepsemis1,fepsbuoy1
      integer qiyr(0:mxdur,mxfire)
      integer qijul(0:mxdur,mxfire)
      integer qihr(0:mxdur,mxfire)
      integer qidur(0:mxdur,mxfire)
      real qxheat(0:mxdur,mxfire)
      real qxflux(0:mxdur,mxfire)
      real qxarea(0:mxdur,mxfire)
      real qxtambk(0:mxdur,mxfire)
      real qemit(0:mxdur,mxspec,mxfire)
      integer idurtot

c --- Initialization
      ctmp(1:1)=' '
      blnk(1:1)=' '
      cevnt1(1:1)=' '
      cevnt2(1:1)=' '
      do i=1,20
        qctmp20(i)(1:1)=' '
      enddo
      do i=0,mxdur
        qiyr(i,ifire)=0
        qijul(i,ifire)=0
        qihr(i,ifire)=0
        qidur(i,ifire)=0
        qxheat(i,ifire)=0.
        qxflux(i,ifire)=0.
        qxarea(i,ifire)=0.
        qxtambk(i,ifire)=0.
        do j=1,nspec
          qemit(i,j,ifire)=0.
        end do
      end do
      imo=0
      iday=0
      idurtot=0

c --- Open the FEPS Consumption/Emissions Report file
      open(unit=iofep1,file=fepsemis1,status='old')

c --- Open the FEPS Buoyancy Results Report file
      open(unit=iofep2,file=fepsbuoy1,status='old')

c --- Read FEPS Consumption/Emissions Report header line 1
      read(iofep1,'(a132)')ctmp

c --- Parse the line to confirm the file is FEPS Consumption/Emissions
      call PARSE(ctmp,',',20,qctmp20)
      ctmp=qctmp20(1)
      call PARSE(ctmp,'-',2,qctmp20)
      ctmp=qctmp20(2)
      ctmp=TRIM(ADJUSTL(ctmp))
      if(ctmp.NE.'Consumption / Emissions Report') then
        write(iolst,*) 'READFEPS Error:'
        write(iolst,*) ' ',fepsemis1
        write(iolst,*) ' is not a FEPS Consumption/Emissions Report'
        goto 99
      endif

c --- Read FEPS Buoyancy Results Report header line 1
      read(iofep2,'(a132)')ctmp
c --- Parse the line to confirm the file is FEPS Buoyancy Results
      call PARSE(ctmp,',',20,qctmp20)
      ctmp=qctmp20(1)
      call PARSE(ctmp,'-',2,qctmp20)
      ctmp=qctmp20(2)
      ctmp=TRIM(ADJUSTL(ctmp))
      if(ctmp.NE.'Bouyancy Results') then
        write(iolst,*) 'READFEPS Error:'
        write(iolst,*) ' ',fepsbuoy1
        write(iolst,*) ' is not a FEPS Buoyancy Results Report'
        goto 99
      endif

c --- Read past header line 2 in both FEPS files
      read(iofep1,*)
      read(iofep2,*)

c --- Read header line 3 and extract Event name in FEPS C/E Report
      read(iofep1,'(a132)')ctmp
      call PARSE(ctmp,',',20,qctmp20)
      cevnt1=qctmp20(2)

c --- Read header line 3 and extract Event name in FEPS BUOY Report
      read(iofep2,'(a132)')ctmp
      call PARSE(ctmp,',',20,qctmp20)
      cevnt2=qctmp20(2)

c --- Check that the Event Names match across the two FEPS files
      if(cevnt1.NE.cevnt2) then
        write(iolst,*) 'READFEPS Error:'
        write(iolst,*) ' FEPS Event Names do not match between '
        write(iolst,*) ' ',fepsemis1
        write(iolst,*) ' and'
        write(iolst,*) ' ',fepsbuoy1
        goto 99
      else
        cevnt=cevnt1
      endif

c --- Read past header line 4 in both FEPS files
      read(iofep1,*)
      read(iofep2,*)

c --- Read header line 5 and extract Temperature column label in FEPS C/E Report
      read(iofep1,'(a132)')ctmp
      call PARSE(ctmp,',',20,qctmp20)
      ctmp=qctmp20(4)
c --- Confirm units are metric
      if(ctmp.NE.'Temperature (C)') then
        write(iolst,*) 'READFEPS Error:'
        write(iolst,*) ' ',fepsemis1
        write(iolst,*) ' is not in metric units'
        goto 99
      endif

c --- Read header line 5 and extract Temperature column label in FEPS BUOY Report
      read(iofep2,'(a132)')ctmp
      call PARSE(ctmp,',',20,qctmp20)
      ctmp=qctmp20(4)
c --- Confirm units are metric
      if(ctmp.NE.'Temperature (C)') then
        write(iolst,*) 'READFEPS Error:'
        write(iolst,*) ' ',fepsbuoy1
        write(iolst,*) ' is not in metric units'
        goto 99
      endif

c --- Read past header line 6 in both FEPS files
      read(iofep1,*)
      read(iofep2,*)

c --- Skip first record where Elapsed Time = 0 hrs in both FEPS files
      read(iofep1,*)
      read(iofep2,*)

c -----------------------------------------------------------
c --- Loop over each record in both FEPS files simultaneously
c -----------------------------------------------------------
c --- Initialize array index
      i=1

10    continue

c --- Read record in FEPS C/E Report File
      read(iofep1,'(a132)',end=90)ctmp
      call PARSE(ctmp,',',20,qctmp20)

c --- Extract duration (hours)
      ctmp=TRIM(ADJUSTL(qctmp20(1)))
c --- Convert from string to integer
      read(ctmp,'(i3)') qidur(i,ifire)

c --- Extract time
      ctmp=TRIM(ADJUSTL(qctmp20(2)))
      call PARSE(ctmp,'/',3,qctmp3)
      ctmp=TRIM(ADJUSTL(qctmp3(1)))
      read(ctmp,'(i2)') imo
      ctmp=TRIM(ADJUSTL(qctmp3(2)))
      read(ctmp,'(i2)') iday
      ctmp=TRIM(ADJUSTL(qctmp3(3)))
      read(ctmp,'(i4)') qiyr(i,ifire)
c --- Convert Gregorian Date to Julian Day
      call JULDAY(iolst,qiyr(i,ifire),imo,iday,qijul(i,ifire))
c --- Get hour and convert from string to integer
      ctmp=TRIM(ADJUSTL(qctmp20(3)))
      read(ctmp,'(i2)') qihr(i,ifire)

c --- Extract ambient air temperature (C)
      ctmp=TRIM(ADJUSTL(qctmp20(4)))
c --- Convert from string to real      
      call RGET132(ctmp,1,INDEX(ctmp,' '),qxtambk(i,ifire))

c --- Extract area (hectares)
      ctmp=TRIM(ADJUSTL(qctmp20(6)))
c --- Convert from string to real      
      call RGET132(ctmp,1,INDEX(ctmp,' '),qxarea(i,ifire))

c --- Extract CO Emissions (g/s)
      ctmp=TRIM(ADJUSTL(qctmp20(12)))
c --- Convert from string to real      
      call RGET132(ctmp,1,INDEX(ctmp,' '),qemit(i,1,ifire))

c --- Extract CH4 Emissions (g/s)
      ctmp=TRIM(ADJUSTL(qctmp20(13)))
c --- Convert from string to real      
      call RGET132(ctmp,1,INDEX(ctmp,' '),qemit(i,2,ifire))

c --- Extract PM2.5 Emissions (g/s)
      ctmp=TRIM(ADJUSTL(qctmp20(14)))
c --- Convert from string to real      
      call RGET132(ctmp,1,INDEX(ctmp,' '),qemit(i,3,ifire))


c --- Read record in FEPS Buoy Report File
      read(iofep2,'(a132)',end=90)ctmp
      call PARSE(ctmp,',',20,qctmp20)

c --- Extract Plume Heat Release (i.e. heat emission) (kJ/hr)
      ctmp=TRIM(ADJUSTL(qctmp20(13)))
c --- Convert from string to real
      call RGET132(ctmp,1,INDEX(ctmp,' '),qxheat(i,ifire))

c --- Extract Briggs Buoyancy (i.e. plume buoyancy flux) (m**4/s**3)
      ctmp=TRIM(ADJUSTL(qctmp20(14)))
c --- Convert from string to real
      call RGET132(ctmp,1,INDEX(ctmp,' '),qxflux(i,ifire))

c --- Increment array index
      i=i+1

      goto 10

90    continue

c --- Extract total duration of fire event
      idurtot=qidur(i-1,ifire)

      close(iofep1)
      close(iofep2)

      return

c --- STOP now if error has occurred
99    write(iomsg,*) 'ERROR has occurred:'
      write(iomsg,*) '  Review messages written to the LIST file'
      stop

      end

c----------------------------------------------------------------------
      subroutine AREATOSQ(area,xcenter,ycenter,vertx,verty,radi)
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43          Level: 070522          AREATOSQ
c                I. Lee, TRC Env. Corp.
c
c --- PURPOSE:   Compute coordinates for a square (km) with area 
c                equivalent to input area (hectares) as well as
c                source radius (m) used in computing flame height.
c
c --- UPDATES:
c
c --- INPUTS:    AREA     - real     - area of fire (hectares)
c                XCENTER  - real     - X coord. (km) of center of fire
c                                      in user's map projection
c                YCENTER  - real     - Y coord. (km) of center of fire
c                                      in user's map projection
c
c --- OUTPUTS:   VERTX(4) - real     - array of X coords. of square (km) 
c                                      with area equivalent to input 
c                                      acreage in user's map projection
c                VERTY(4) - real     - array of Y coords. of square (km) 
c                                      with area equivalent to input 
c                                      acreage in user's map projection
c                RADI     - real     - source radius (m)
c
c --- PARAMETERS USED:
c                None
c
c --- COMMON BLOCK VARIABLES USED:
c                None
c
c --- CALLED BY: COMP
c
c --- CALLS:     None
c
c----------------------------------------------------------------------
c
c --- Local variables
      real vertx(4),verty(4)
      real halfside
      real radi

c --- Set constants
      data pi/3.1415926/

c --- Convert area from (hectares) to (m**2)
      area=area*10000

c --- Compute half-side of equivalent area square (km)
      halfside=0.001*sqrt(area)/2.

c --- Compute coordinates (km) of square
      vertx(1)=xcenter-halfside
      vertx(2)=xcenter+halfside
      vertx(3)=xcenter+halfside
      vertx(4)=xcenter-halfside
      verty(1)=ycenter+halfside
      verty(2)=ycenter+halfside
      verty(3)=ycenter-halfside
      verty(4)=ycenter-halfside

c --- Compute source radius (m)
      radi=sqrt(area/pi)

      return
      end

c----------------------------------------------------------------------
      subroutine HFLAME(h,q,d,ramb0,tamb0,cplume,gravi)
c----------------------------------------------------------------------
C
c --- FEPS2BAEM  Version: 1.43          Level: 070522            HFLAME
C                Xiaoming (John) Zhang
C                Earth Tech, Inc.
C
C     This subroutine calculates the flame height based on:
C     Cetegen, B.M., Zukoski, E.E., and Kubota, T.,
C     Entrainment and flame geometry of fire plumes, Report NBS-GCR-82-402,
C     Center for fire research, National Bureau of Standards, Gaithersburg,
C     MD (August 1982).
C
c --- UPDATES:
c
C --- INPUTS:
C       Q:      Heat release rate                              kW=kJ/s
C       D:      Diameter of the fire source,                   m
C
C       The following parameters had better be input from users:
C       RAMB0:  Ground air density at ambient temperature,     kg/m**3
C       TAMB0:  Ground ambient temperature,                    K
C       CPLUME: Specific heat of plume,                        kJ/(kg*K)
C       GRAVI:  Gravitational acceleration                     m/s**2
C
C --- OUTPUTS:
C       H:      Flame height,                                  m
C
c --- PARAMETERS USED:
c                None
c
c --- COMMON BLOCK VARIABLES USED:
c                None
c
c --- CALLED BY: COMP
c
c --- CALLS:     None
c
c----------------------------------------------------------------------
c ---  Non-dimensional heat flux
       qstar=q/(ramb0*cplume*tamb0*gravi**0.5*d**2.5)
c
       if (qstar.le.0.865) then
         h=3.18*d*qstar**0.4
       else
         h=3.30*d*qstar**0.744
       end if
c
       return
       end

c----------------------------------------------------------------------
      subroutine EFFWP(fb,tambk,rplume,tplumek,grav,wplume)
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43           Level: 070522            EFFWP
c                I. Lee, TRC Env. Corp.
c
c --- PURPOSE:   Calculate effective plume vertical velocity (m/s)
c
c --- UPDATES:
c
c --- INPUTS:
c           FB   - real    - plume buoyancy flux (m**4/s**3)
c        TAMBK   - real    - ambient air temperature (K)
c       RPLUME   - real    - plume radius (m)
c      TPLUMEK   - real    - plume temperature (K)
c         GRAV   - real    - gravity (m/s**2)
c
c --- OUTPUTS:
c       WPLUME   - real    - plume vertical velocity (m/s)
c
c --- PARAMETERS USED:
c                None
c
c --- COMMON BLOCK VARIABLES USED:
c                None
c
c --- CALLED BY: COMP
c
c --- CALLS:     None
c
c----------------------------------------------------------------------
c
c --- Local variables
      real fb,tambk,rplume,tplumek,grav,wplume

c --- Compute plume vertical velocity (m/s)
      wplume=(fb*tplumek)/(grav*(rplume**2)*(tplumek-tambk))

      return
      end

c----------------------------------------------------------------------
      subroutine PMDIST4(pm2p5,ncats,subcats)
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43             Level: 070613      PMDIST4
c                I. Lee, TRC Env. Corp.
c
c --- PURPOSE:   Calculate PM2.5 sub-size categories
c                  PM1.87 = 62.50% * PM2.5
c                  PM1.12 = 12.50% * PM2.5
c                  PM0.81 = 18.75% * PM2.5
c                  PM0.56 =  6.25% * PM2.5
c
c --- UPDATES:
c
c --- INPUTS:
c       PM2P5        - real    - PM2.5 emission rate (g/s)
c       NCATS        - integer - number of sub-size categories
c
c --- OUTPUTS:
c       SUBCATS(4)   - real    - PM1.87 emission rate (g/s)
c       SUBCATS(5)   - real    - PM1.12 emission rate (g/s)
c       SUBCATS(6)   - real    - PM0.81 emission rate (g/s)
c       SUBCATS(7)   - real    - PM0.56 emission rate (g/s)
c
c --- PARAMETERS USED:
c                None
c
c --- COMMON BLOCK VARIABLES USED:
c                None
c
c --- CALLED BY: COMP
c
c --- CALLS:     None
c
c----------------------------------------------------------------------
c
c --- Local variables
      real pm2p5,subcats(ncats)

      subcats(1)=0.625*pm2p5
      subcats(2)=0.125*pm2p5
      subcats(3)=0.1875*pm2p5
      subcats(4)=0.0625*pm2p5

      return
      end

c----------------------------------------------------------------------
      subroutine PARSE(cstr,delim,mxelem,carray)
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43         Level: 070522              PARSE
c                I. Lee, TRC Env. Corp.
c
c --- PURPOSE:   Parse delimited string into elements of an array
c
c --- UPDATES:
c
c --- INPUTS:    CSTR            - character  - input string (132 chars)
c                DELIM           - character  - delimiter
c                MXELEM          - integer    - max number of elements
c                                               in CARRAY
c
c --- OUTPUTS:   CARRAY(MXELEM)  - character  - array of parsed elements
c                                               where each element can
c                                               be up to 132 characters
c
c --- PARAMETERS USED:
c                None
c
c --- COMMON BLOCK VARIABLES USED:
c                None
c
c --- CALLED BY: READFEPS
c
c --- CALLS:     None
c
c----------------------------------------------------------------------
c
c --- Local variables
      character*1 delim
      character*132 cstr
      character*132 carray(mxelem)
      integer idelim

c --- Initialize array
      do i=1,mxelem
        carray(i)(1:1)=' '
      enddo

c --- Loop over max number of elements in array
      do i=1,mxelem
c ---   Get index of first delimiter
        idelim=INDEX(cstr,delim)
c ---   If no delimiter is found
        if(idelim.EQ.0) then
          carray(i)=cstr
          goto 10
        else
c ---     Put string up to first delimiter into array
          carray(i)=cstr(1:idelim-1)
c ---     Remove part of string up to and including first delimiter
          cstr=cstr(idelim+1:132)
        endif
      enddo

10    continue

      return
      end

c----------------------------------------------------------------------
      subroutine RGET132(c132,i1,i2,x)
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43         Level: 060922            RGET132
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert character string to a real.  This sub is needed
c               when there is no decimal point.  The LF95 compiler
c               does not do the internal read conversion without the '.'
c               Also, determine when exponential notation is present.
c
c --- INPUTS:
c             c132 - character*132     - Input character string
c               i1 - integer           - First character of substring
c               i2 - integer           - Last character of substring
c
c --- OUTPUT:
c                X - real              - Output real
c
c --- PARAMETERS USED:
c                None
c
c --- COMMON BLOCK VARIABLES USED:
c                None
c
c --- CALLED BY: READFEPS
c
c --- CALLS:     None
c
c----------------------------------------------------------------------
      character*132 c132
      character*20 c20, blnk20
      data blnk20/'                    '/

      if(i1.GT.i2) return

      c20=blnk20
      c20=c132(i1:i2)

c --- Left justify field
      c20=ADJUSTL(c20)

c --- Test for exponential notation
      ie=MAX(INDEX(c20,'e'),INDEX(c20,'E'))

      if(ie.EQ.0) then
c ---    Simple real number
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            ipt=LEN_TRIM(c20)+1
            c20(ipt:ipt)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(f)') x

      else
c ---    Exponential notation
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            do k=20,ie+1,-1
               km1=k-1
               c20(k:k)=c20(km1:km1)
            enddo
            c20(ie:ie)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(e)') x

      endif

      return
      end

c----------------------------------------------------------------------
      subroutine FIN
c----------------------------------------------------------------------
c
c --- FEPS2BAEM  Version: 1.43          Level: 090511               FIN
c ---            J. Scire
c
c --- PURPOSE:  Run termination routine -- compute runtime
c
c --- UPDATES
c
c --- V1.0 (070522) to 1.42 (090511) (DGS)
c              - Reformat date reported at end of run
c
c --- PARAMETERS USED:
c                FEPS2BAEM Parameters
c                iolst, iomsg
c
c --- COMMON BLOCK VARIABLES USED:
c                /QA/ Common Block
c                rcpu, rdate, rtime
c
c --- CALLED BY: MAIN
c
c --- CALLS:     DATETM, JULDAY, DELTT, FMT_DATE
c----------------------------------------------------------------------
c
c --- include parameters
      include 'params.f2b'
c
c --- include common blocks
      include 'qa.f2b'
c
      character*8 rtime2
      character*10 rdate2
      character*12 rdate12
c
      write(iomsg,*)'TERMINATION PHASE'
c
c --- get system date & time at end of run
      call DATETM(rdate2,rtime2,rcpu)
c
c --- compute runtime
      read(rtime(1:2),10)ihr1
      read(rtime(4:5),10)imin1
      read(rtime(7:8),10)isec1
10    format(i2)
      t1=ihr1*3600.+imin1*60.+isec1
c
      read(rtime2(1:2),10)ihr2
      read(rtime2(4:5),10)imin2
      read(rtime2(7:8),10)isec2
      t2=ihr2*3600.+imin2*60.+isec2
c
      if(rdate.eq.rdate2)then
         delt=t2-t1
      else
         read(rdate(1:2),10)imo1
         read(rdate(4:5),10)iday1
         read(rdate(7:10),'(i4)')iyr1
         call JULDAY(iolst,iyr1,imo1,iday1,ijul1)
c
         read(rdate2(1:2),10)imo2
         read(rdate2(4:5),10)iday2
         read(rdate2(7:10),'(i4)')iyr2
         call JULDAY(iolst,iyr2,imo2,iday2,ijul2)
c
c ---    compute no. hours from beg. of first hour of run to
c ---    ending hour of ending day of the run
         call DELTT(iyr1,ijul1,ihr1,iyr2,ijul2,ihr2,idelhr)
c
c ---    adjust for minutes and seconds
         delt=idelhr*3600.-imin1*60.-isec1+imin2*60.+isec2
      endif

c --- On the PC, the runtime and CPU time are the same
c --- (DATETM provides RCPU = 0.0 on the PC)
      if(rcpu.EQ.0.0)rcpu=delt

c --- Report current date
      rdate12=rdate2(1:10)//'  '
      call FMT_DATE(iolst,'MM-DD-YYYY','DD-MMM-YYYY',rdate12)
      write(iolst,1402)rtime2,rdate12,NINT(delt),NINT(rcpu)
1402  format(//2x,'End of run -- Clock time: ',a8/
     1         2x,'                    Date: ',a12//
     2         2x,'      Elapsed Clock Time: ',i12,' (seconds)'//
     3         2x,'                CPU Time: ',i12,' (seconds)')

c
      return
      end
